import math
import imp
import time
imp.load_source("util","ScriptEditor/util.py")  
from util import *
imp.load_source("dynamicBoundary","ScriptEditor/dynamicBoundary.py")  
from dynamicBoundary import *


translator.install()
			
	
def computeDynamic(simulator,\
					currentGeometry,\
					currentFill,\
					currentBoundary,\
					mainSimulation,\
					mainSimulationDescriptors,\
					Input,\
					Output,\
					OutputDescriptors,\
					boundaryGroups,\
					timingDID,\
					startTime,\
					endTime,\
					offset):

	simulationSettings = simulator.getSimulationSettings(currentGeometry, currentFill, currentBoundary)
	transientSimulationSettings = simulationSettings["transient"]
	precision = transientSimulationSettings["precision"]
	simulateFlows = "compute_flows" in transientSimulationSettings and transientSimulationSettings["compute_flows"]
	simulateDew = "compute_dew" in transientSimulationSettings and transientSimulationSettings["compute_dew"]
	simulateCondensation = "compute_condensation" in transientSimulationSettings and transientSimulationSettings["compute_condensation"]
		
	if precision == "low":
		numSteps = 200
		maxNumSavedSteps = 200
		numSubSteps = 1
	elif precision == "medium":
		numSteps = 600
		maxNumSavedSteps = 400
		numSubSteps = 1
	elif precision == "high":
		numSteps = 5000
		maxNumSavedSteps = 800
		numSubSteps = 1
	elif precision == "very_high":
		if simulateCondensation:
			numSteps = 15000
			maxNumSavedSteps = 2000
			numSubSteps = 1
		else:
			numSteps = 5000
			maxNumSavedSteps = 1000
			numSubSteps = 3

	stepsToSave = numSteps
	stepsToSaveSkip = 1	
	while stepsToSave > maxNumSavedSteps:
		stepsToSaveSkip = stepsToSaveSkip + 1
		stepsToSave = numSteps / stepsToSaveSkip
		
	if offset < 0:
		offset = 0
	
	if timingDID == "__dynamic_timing_month__":
		startMonth = startTime - 1
		if startMonth < 0:
			startMonth = 0
		if startMonth > 11:
			startMonth = 11
		if offset < 0:
			offset = 0
		if offset > 12:
			offset = 12
		startMonthWithOffset = startMonth - offset
		if startMonthWithOffset < 0:
			simulationStartTime = getMonthTime(startMonthWithOffset + 12) - getMonthTime(12)
		else:
			simulationStartTime = getMonthTime(startMonthWithOffset)
		offset = getMonthTime(startMonth) - simulationStartTime
		
		endMonth = endTime
		if endMonth < 0:
			endMonth = 0
		#if endMonth > 12:
		#	endMonth = 12
			
		simulationEndTime = getMonthTime(endMonth)	
	else:
		if timingDID == "__dynamic_timing_minute__":
			timeFactor = 60.0
		elif timingDID == "__dynamic_timing_hour__":
			timeFactor = 60.0 * 60.0
		elif timingDID == "__dynamic_timing_day__":
			timeFactor = 60.0 * 60.0 * 24.0
		else:			
			timeFactor = 1.0
			
		simulationStartTime = startTime * timeFactor - offset * timeFactor
		simulationEndTime = endTime * timeFactor
		offset = offset * timeFactor
				
	stepLength = (simulationEndTime - simulationStartTime) / numSteps

	dynamicSimulationNames = list()
	dynamicSimulationTags = list()
	dynamicVariants = list()
	dynamicSpecialElementsConfigurations = list()
	
	if simulateFlows:
		dynamicSimulationNames.append(_("Dynamic flows"))
		dynamicSimulationTags.append("d_flow_s")
		dynamicVariants.append("flows")
		dynamicSpecialElementsConfigurations.append(list())
	if simulateDew:
		dynamicSimulationNames.append(_("Dynamic Surf. Cond."))
		dynamicSimulationTags.append("d_dew_s")
		dynamicVariants.append("dew")
		# Tell simulator to perform analysis on section elements
		dynamicSpecialElementsConfigurations.append([2])
	if simulateCondensation:
		dynamicSimulationNames.append(_("Dynamic mold"))
		dynamicSimulationTags.append("d_cond_s")
		dynamicVariants.append("mold")
		# Tell simulator to perform analysis on section elements
		dynamicSpecialElementsConfigurations.append([2])
		
	if len(dynamicSimulationNames) == 0:
		return

	Input["progress_bar_multiplier"] = (numSteps + 1) * len(dynamicSimulationNames)
	
	for currentSimulation in range(len(dynamicSimulationNames)):
		specialElementsConfig = dynamicSpecialElementsConfigurations[currentSimulation]
		Input["dynamic"] = False
		Input["static_for_dynamic"] = True
		Input["time"] = simulationStartTime		
		Input["boundary_variant"] = dynamicVariants[currentSimulation]
		Input["offset_simulation"] = offset > 0.0
		
		monthlyCondensationGrid = dynamicVariants[currentSimulation] == "mold" and timingDID == "__dynamic_timing_month__"
		
		boundaryConditions = simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)
		
		startTime = time.time()
		
		simulationOutput, simulationOutputDescriptors = addSimulation(simulator,\
																		currentGeometry,\
																		currentFill,\
																		currentBoundary,\
																		specialElementsConfig,\
																		boundaryConditions,\
																		Input,\
																		Output,\
																		OutputDescriptors,\
																		dynamicSimulationNames[currentSimulation],\
																		dynamicSimulationTags[currentSimulation])

		if monthlyCondensationGrid:
			dynamicCondensationGridDescriptor = createEmptyGridDescriptor("dyn_cond_grid", _("Monthly condensation summary"), "d_c_g", 600, -1)
			addGridColStringDescriptor(dynamicCondensationGridDescriptor, "period", _("Period"))
			addGridColNumberDescriptor(dynamicCondensationGridDescriptor, "t", _("Time"), "h", "%.2f")
			addGridColNumberDescriptor(dynamicCondensationGridDescriptor, "t_p", _("Time"), "%", "%.1f")
			addGridColNumberDescriptor(dynamicCondensationGridDescriptor, "t_x", _("Time") + " > TX", "h", "%.2f")
			addGridColNumberDescriptor(dynamicCondensationGridDescriptor, "t_x_p", _("Time") + " > TX", "%", "%.1f")
			#addGridColNumberDescriptor(dynamicCondensationGridDescriptor, "t_y", _("Time") + " < TY", "h", "%.2f")
			#addGridColNumberDescriptor(dynamicCondensationGridDescriptor, "t_y_p", _("Time") + " < TY", "%", "%.1f")
			
			dynamicCondensationGrid = list()
			simulationOutputAdvancedResults = simulationOutput["advanced_results"]
			simulationOutputBoundary = simulationOutputAdvancedResults["boundary"]
			simulationOutputBoundary[dynamicCondensationGridDescriptor["bind"]] = dynamicCondensationGrid
			
			simulationOutputAdvancedResultsDescriptors = findSubDescriptors("advanced_results", simulationOutputDescriptors)
			simulationOutputBoundaryDescriptors = findSubDescriptors("boundary",  simulationOutputAdvancedResultsDescriptors)
			simulationOutputBoundaryDescriptors.append(dynamicCondensationGridDescriptor)
			
			dynamicCondensationGridRowMap = dict()
			lastCondensationStartTime = simulationStartTime + offset
			lastStepCondensation = False
			if 'mold_x' in transientSimulationSettings:
				condensationXTime = transientSimulationSettings['mold_x'] * 3600.0
			else:
				condensationXTime = 1.0 * 3600.0
			totalCondensationTime = 0.0
			totalCondensationTimeX = 0.0
																		
		addDynamicBoundaryXYZFactorCurves(boundaryConditions, simulationOutput, simulationOutputDescriptors)														

		simulationOutput["dynamic_steps"] = list()
		staticAndDynamicAnalysis = simulationOutput["advanced_results"]["static_and_dynamic_global_analysis"]
		bakedBoundaryConditions = simulationOutput["advanced_results"]["boundary"]["boundaries_complete"]

		staticAndDynamicAnalysisSteps = 1
		
		danamicBoundaryCurves = list()
		dynamicBoundaryTemperatureAnalysis = list()
		dynamicBoundaryGroupTemperatureAnalysis = list()
		dynamicBoundaryFlowAnalysis = list()
		dynamicBoundaryGroupFlowAnalysis = list()
		dynamicBoundaryCondensationAnalysis = list()
		dynamicBoundaryGroupCondensationAnalysis = list()
		
		Input["dynamic_time_step"] = stepLength
		
		currentTime = simulationStartTime + stepLength
		
		for step in range(numSteps):
			
			specialElementsConfig = list()
			Input["dynamic"] = True
			Input["static_for_dynamic"] = False
			Input["new_boundary_conditions"] = False
			Input["dynamic_sub_steps"] = numSubSteps
			Input["time"] = currentTime
			Input["offset_simulation"] = currentTime < simulationStartTime + offset

			dynamicOutput = dict()
			dynamicOutputDescriptors = list()
			stepSimulationOutput, stepSimulationOutputDescriptors = addSimulation(simulator,\
																				currentGeometry,\
																				currentFill,\
																				currentBoundary,\
																				specialElementsConfig,\
																				boundaryConditions,\
																				Input,\
																				dynamicOutput,\
																				dynamicOutputDescriptors,\
																				_("Dynamic step"))
																				
			Input["progress_bar_multiplier"] = 0																	
			simulation_3d = "3d_simulation" in stepSimulationOutput and stepSimulationOutput["3d_simulation"]
			
			if monthlyCondensationGrid and currentTime >= simulationStartTime + offset:
				month = getMonth(currentTime)
				if month in dynamicCondensationGridRowMap:
					newMonth = False
				else:
					dynamicCondensationGridRow = dict()
					dynamicCondensationGridRow['period'] = getMonthName(month)
					dynamicCondensationGridRow['t'] = 0.0
					dynamicCondensationGridRow['t_p'] = 0.0
					dynamicCondensationGridRow['t_x'] = 0.0
					dynamicCondensationGridRow['t_x_p'] = 0.0
					
					dynamicCondensationGridRowMap[len(dynamicCondensationGrid)] = dynamicCondensationGridRow					
					dynamicCondensationGrid.append(dynamicCondensationGridRow)
					newMonth = True
				
				dynamicCondensationGridRow = dynamicCondensationGridRowMap[month]
				
				boundaryAnalysis = stepSimulationOutput["advanced_results"]["boundary"]["tf_analysis"]
				boundaryDew = 0.0
				boundaryMold = 0.0
				for boundaryID in range(len(boundaryConditions)):
					dimensionBind = getTFAnalysisDimensionBind(boundaryAnalysis[boundaryID])
					boundaryDew = boundaryDew + boundaryAnalysis[boundaryID]["dew_" + dimensionBind]
					boundaryMold = boundaryMold + boundaryAnalysis[boundaryID]["mold_" + dimensionBind]
					
				condensationStopped = (step == numSteps - 1) and lastStepCondensation
				monthDuration = getMonthDuration(month) / 3600

				if boundaryDew + boundaryMold > 0.0:
					stepCondensationTime = stepLength
					if newMonth:
						if month == 0:
							newMonthCondensationTime = currentTime - (simulationStartTime + offset)
						else:
							newMonthCondensationTime = currentTime - getMonthTime(month)
						oldMonthCondensationTime = stepCondensationTime - newMonthCondensationTime
						stepCondensationTime = newMonthCondensationTime
						if month > 0:
							oldDynamicCondensationGridRow = dynamicCondensationGridRowMap[month - 1]
							oldMonthDuration = getMonthDuration(month - 1) / 3600.0
							oldDynamicCondensationGridRow['t'] = min(oldMonthDuration, oldDynamicCondensationGridRow['t'] + oldMonthCondensationTime / 3600.0)
							#oldDynamicCondensationGridRow['t'] = oldDynamicCondensationGridRow['t'] + oldMonthCondensationTime / 3600.0
							oldDynamicCondensationGridRow['t_p'] = oldDynamicCondensationGridRow['t'] / oldMonthDuration
							totalCondensationTime = totalCondensationTime + oldMonthCondensationTime
					
					dynamicCondensationGridRow['t'] = min(monthDuration, dynamicCondensationGridRow['t'] + stepCondensationTime / 3600.0)
					#dynamicCondensationGridRow['t'] = dynamicCondensationGridRow['t'] + stepCondensationTime / 3600.0
					dynamicCondensationGridRow['t_p'] = dynamicCondensationGridRow['t'] / monthDuration
					totalCondensationTime = totalCondensationTime + stepCondensationTime
					if not lastStepCondensation:
						lastCondensationStartTime = currentTime - stepLength
					lastStepCondensation = True
				else:
					if lastStepCondensation:
						condensationStopped = True
					else:
						lastCondesationEndTime = currentTime
					lastStepCondensation = False
					
				if condensationStopped:
					totalCondensationPeriod = currentTime - lastCondensationStartTime - stepLength
					if totalCondensationPeriod >= condensationXTime:
						condensationStartMonth = getMonth(lastCondensationStartTime)
						condensationEndMonth = getMonth(currentTime - stepLength)
						condensationStartTime = lastCondensationStartTime
						for condensationMonth in range(condensationStartMonth, condensationEndMonth+1):
							if condensationMonth < condensationEndMonth:
								monthEndTime = getMonthTime(condensationMonth) + getMonthDuration(condensationMonth)
								monthCondensationPeriod = monthEndTime - max(getMonthTime(condensationMonth), condensationStartTime)
								condensationStartTime = monthEndTime									
							else:
								monthCondensationPeriod = currentTime - max(getMonthTime(condensationMonth), condensationStartTime)
								if step == numSteps - 1:
									monthCondensationPeriod = monthCondensationPeriod
								else:
									monthCondensationPeriod = monthCondensationPeriod - stepLength
							
							dynamicCondensationGridRow = dynamicCondensationGridRowMap[condensationMonth]
							condensationMonthDuration = getMonthDuration(condensationMonth) / 3600;
							dynamicCondensationGridRow['t_x'] = min(condensationMonthDuration, dynamicCondensationGridRow['t_x'] + monthCondensationPeriod / 3600.0)
							#dynamicCondensationGridRow['t_x'] = dynamicCondensationGridRow['t_x'] + monthCondensationPeriod / 3600.0
							dynamicCondensationGridRow['t_x_p'] = dynamicCondensationGridRow['t_x'] / condensationMonthDuration
							totalCondensationTimeX = totalCondensationTimeX + monthCondensationPeriod
					lastCondensationStartTime = currentTime
			
			if currentTime >= simulationStartTime + offset and step % stepsToSaveSkip == 0:
				# Store only necessary data in output
				minimalOutput = dict()					
				minimalOutput["simulation_result"] = stepSimulationOutput["simulation_result"]
				minimalOutput["simulation_analysis"] = stepSimulationOutput["simulation_analysis"]
				simulationOutput["dynamic_steps"].append(minimalOutput)
				
				boundaryAnalysis = stepSimulationOutput["advanced_results"]["boundary"]["tf_analysis"]
				boundariesAtSimulationTime = stepSimulationOutput["advanced_results"]["boundary"]["boundaries_at_simulation_time"]
								
				addBoundariesCurvesStep(bakedBoundaryConditions,\
										boundariesAtSimulationTime,\
										currentTime,\
										boundaryAnalysis,\
										danamicBoundaryCurves)
								
				addBoundaryDynamicTemperatureAnalysisStep(currentTime,\
															bakedBoundaryConditions,\
															boundaryGroups,\
															boundaryAnalysis,\
															dynamicBoundaryTemperatureAnalysis,\
															dynamicBoundaryGroupTemperatureAnalysis)

				addBoundaryDynamicFlowAnalysisStep(currentTime,\
													stepLength * stepsToSaveSkip, \
													bakedBoundaryConditions,\
													boundaryGroups,\
													boundaryAnalysis,\
													dynamicBoundaryFlowAnalysis,\
													dynamicBoundaryGroupFlowAnalysis)
															
				addBoundaryDynamicCondensationAnalysisStep(currentTime,\
															bakedBoundaryConditions,\
															boundaryGroups,\
															boundaryAnalysis,\
															dynamicBoundaryCondensationAnalysis,\
															dynamicBoundaryGroupCondensationAnalysis,\
															simulation_3d)
															
				combineAnalysis(staticAndDynamicAnalysis, stepSimulationOutput["advanced_results"]["static_and_dynamic_global_analysis"])
				staticAndDynamicAnalysisSteps = staticAndDynamicAnalysisSteps + 1
															
			currentTime = currentTime + stepLength

		if monthlyCondensationGrid:
			dynamicCondensationGridRow = dict()
			dynamicCondensationGridRow['period'] = _('Total')
			dynamicCondensationGridRow['t'] = totalCondensationTime / 3600.0
			dynamicCondensationGridRow['t_p'] = dynamicCondensationGridRow['t'] / (getYearDuration() / 3600.0)
			dynamicCondensationGridRow['t_x'] = totalCondensationTimeX / 3600.0
			dynamicCondensationGridRow['t_x_p'] = dynamicCondensationGridRow['t_x'] / (getYearDuration() / 3600.0)
			dynamicCondensationGrid.append(dynamicCondensationGridRow)
			
		simulationOutput["dynamic_start_time"] = simulationStartTime + offset
		simulationOutput["dynamic_end_time"] = simulationEndTime
			
		addDynamicBoundaryCurves(transientSimulationSettings,\
								simulationOutput,\
								simulationOutputDescriptors,\
								danamicBoundaryCurves)

		addDynamicBoundaryAnalysis(transientSimulationSettings,\
									bakedBoundaryConditions,\
									boundaryGroups,\
									simulationOutput,\
									simulationOutputDescriptors,\
									dynamicBoundaryTemperatureAnalysis,\
									dynamicBoundaryGroupTemperatureAnalysis,\
									"temp")	
									
		addDynamicBoundaryAnalysis(transientSimulationSettings,\
									bakedBoundaryConditions,\
									boundaryGroups,\
									simulationOutput,\
									simulationOutputDescriptors,\
									dynamicBoundaryFlowAnalysis,\
									dynamicBoundaryGroupFlowAnalysis,\
									"flow")
									
		addDynamicBoundaryAnalysis(transientSimulationSettings,\
									bakedBoundaryConditions,\
									boundaryGroups,\
									simulationOutput,\
									simulationOutputDescriptors,\
									dynamicBoundaryCondensationAnalysis,\
									dynamicBoundaryGroupCondensationAnalysis,\
									"cond")
									
		endCombiningAnalysis(staticAndDynamicAnalysis, staticAndDynamicAnalysisSteps - 1)
		
		simulationOutput["sim_time"] = time.time() - startTime
		
